@@ -1,32 +1,25 @@
 mod client_socket_connection;
+mod connection_info;
 mod messages;
 mod room_actor;
-mod room_id;
 mod server_state;
 mod service_actor;
-mod shutdown_policy;
 
-pub use crate::room_id::{
-    RoomIdGenerator, RoomIdStrategy, ShortRoomIdGenerator, ShortRoomIdGeneratorFactory,
-    UuidRoomIdGenerator, UuidRoomIdGeneratorFactory,
-};
+use crate::room_actor::GetConnectionInfo;
 use actix_web::error::ErrorInternalServerError;
-use actix_web::web::{self, get, post};
+use actix_web::web::{self, get};
 use actix_web::{web::Data, App, Error, HttpRequest, HttpResponse, HttpServer, Result};
 use actix_web_actors::ws;
 pub use client_socket_connection::ClientSocketConnection;
-use jamsocket::{JamsocketServiceFactory, SimpleJamsocketService, SimpleJamsocketServiceFactory};
+use connection_info::ConnectionInfo;
+use jamsocket::JamsocketServiceFactory;
 pub use messages::{AssignClientId, MessageFromClient, MessageFromServer};
 pub use room_actor::RoomActor;
 use server_state::ServerState;
 pub use service_actor::{ServiceActor, ServiceActorContext};
-pub use shutdown_policy::ServiceShutdownPolicy;
 use std::time::{Duration, Instant};
 
-#[allow(clippy::unused_async)]
-async fn status() -> Result<HttpResponse, Error> {
-    Ok(HttpResponse::Ok().body("ok"))
-}
+const DEFAULT_IP: &str = "0.0.0.0";
 
 /// Settings used by the server.
 pub struct Server {
@@ -40,14 +33,11 @@ pub struct Server {
     /// Defaults to 5 minutes.
     pub heartbeat_timeout: Duration,
 
-    /// The method by which new rooms are created and assigned names.
-    pub room_id_strategy: RoomIdStrategy,
-
     /// The port to run the server on. Defaults to 8080.
     pub port: u32,
 
-    /// How the server decides to shut down a room once it is empty.
-    pub shutdown_policy: ServiceShutdownPolicy,
+    /// The IP to listen on. Defaults to 0.0.0.0.
+    pub ip: String,
 
     /// A local filesystem path to serve static files from, or None (default).
     pub static_path: Option<String>,
@@ -62,8 +52,7 @@ impl Default for Server {
             heartbeat_interval: Duration::from_secs(30),
             heartbeat_timeout: Duration::from_secs(300),
             port: 8080,
-            room_id_strategy: RoomIdStrategy::default(),
-            shutdown_policy: ServiceShutdownPolicy::Never,
+            ip: DEFAULT_IP.to_string(),
             static_path: None,
             client_path: None,
         }
@@ -109,25 +98,12 @@ impl Server {
     }
 
     #[must_use]
-    pub fn with_room_id_strategy(mut self, room_id_strategy: RoomIdStrategy) -> Self {
-        self.room_id_strategy = room_id_strategy;
+    pub fn with_ip(mut self, ip: String) -> Self {
+        self.ip = ip;
         self
     }
 
-    #[must_use]
-    pub fn with_shutdown_policy(mut self, shutdown_policy: ServiceShutdownPolicy) -> Self {
-        self.shutdown_policy = shutdown_policy;
-        self
-    }
-
-    pub fn serve_default<F: SimpleJamsocketService>(self) -> std::io::Result<()> {
-        let host_factory: SimpleJamsocketServiceFactory<F, ServiceActorContext> =
-            SimpleJamsocketServiceFactory::default();
-
-        self.serve(host_factory)
-    }
-
-    /// Start a server given a cloneable [JamsocketServiceBuilder].
+    /// Start a server given a [JamsocketService].
     ///
     /// This function blocks until the server is terminated. While it is running, the following
     /// endpoints are available:
@@ -135,22 +111,21 @@ impl Server {
     /// - `/new_room` (POST): create a new room, if not in `explicit` room creation mode.
     /// - `/ws/{room_id}` (GET): initiate a WebSocket connection to the given room. If the room
     ///     does not exist and the server is in `implicit` room creation mode, it will be created.
-    pub fn serve<F: JamsocketServiceFactory<ServiceActorContext>>(
+    pub fn serve(
         self,
-        host_factory: F,
+        service_factory: impl JamsocketServiceFactory<ServiceActorContext>,
     ) -> std::io::Result<()> {
-        let host = format!("127.0.0.1:{}", self.port);
-        let server_state = Data::new(ServerState::new(host_factory, self));
+        let host = format!("{}:{}", self.ip, self.port);
 
         actix_web::rt::System::new().block_on(async move {
+            let server_state = Data::new(ServerState::new(service_factory, self).unwrap());
+
             let server = HttpServer::new(move || {
                 #[allow(unused_mut)] // mut only needed with crate feature `serve-static`.
                 let mut app = App::new()
                     .app_data(server_state.clone())
                     .route("/status", get().to(status))
-                    .route("/new_room", post().to(new_room::<F>))
-                    .route("/ws/{room_id}", get().to(websocket::<F>))
-                    .route("/ws/{room_id}", post().to(new_room_explicit::<F>));
+                    .route("/ws", get().to(websocket));
 
                 #[cfg(feature = "serve-static")]
                 {
@@ -165,7 +140,7 @@ impl Server {
                         );
                     }
                 }
-
+    
                 app
             })
             .bind(&host)?;
@@ -176,37 +151,10 @@ impl Server {
     }
 }
 
-async fn new_room<F: JamsocketServiceFactory<ServiceActorContext>>(
-    req: HttpRequest,
-) -> actix_web::Result<HttpResponse> {
-    let server_state: &Data<ServerState<F>> = req.app_data().expect("Could not load ServerState.");
-    let room_id = server_state.new_room_generated().await?;
-
-    Ok(HttpResponse::Ok().body(room_id))
-}
-
-async fn new_room_explicit<F: JamsocketServiceFactory<ServiceActorContext>>(
-    req: HttpRequest,
-    room_id: web::Path<String>,
-) -> actix_web::Result<HttpResponse> {
-    let server_state: &Data<ServerState<F>> = req.app_data().expect("Could not load ServerState.");
-    server_state.explicit_new_room(room_id.as_ref()).await?;
-
-    Ok(HttpResponse::Ok().body(room_id.to_string()))
-}
-
-async fn websocket<F: JamsocketServiceFactory<ServiceActorContext>>(
-    req: HttpRequest,
-    stream: web::Payload,
-    room_id: web::Path<String>,
-) -> actix_web::Result<HttpResponse> {
-    let ip = req
-        .peer_addr()
-        .map_or_else(|| "<unknown>".to_string(), |a| a.ip().to_string());
-
-    let server_state: &Data<ServerState<F>> = req.app_data().expect("Could not load ServerState.");
-    let room_addr = server_state.connect_room(room_id.as_ref()).await?;
+async fn websocket(req: HttpRequest, stream: web::Payload) -> actix_web::Result<HttpResponse> {
+    let server_state: &Data<ServerState> = req.app_data().expect("Could not load ServerState.");
 
+    let room_addr = server_state.room_addr.clone();
     let client_id = room_addr
         .send(AssignClientId)
         .await
@@ -216,8 +164,6 @@ async fn websocket<F: JamsocketServiceFactory<ServiceActorContext>>(
         ClientSocketConnection {
             room: room_addr.clone().recipient(),
             client_id,
-            ip: ip.clone(),
-            room_id: room_id.clone(),
             last_seen: Instant::now(),
             heartbeat_interval: server_state.settings.heartbeat_interval,
             heartbeat_timeout: server_state.settings.heartbeat_timeout,
@@ -227,12 +173,7 @@ async fn websocket<F: JamsocketServiceFactory<ServiceActorContext>>(
         stream,
     ) {
         Ok((addr, resp)) => {
-            tracing::info!(
-                %ip,
-                %room_id,
-                ?client_id,
-                "New connection",
-            );
+            tracing::info!(?client_id, "New connection",);
             room_addr.do_send(MessageFromClient::Connect(client_id, addr.recipient()));
 
             Ok(resp)
@@ -240,3 +181,15 @@ async fn websocket<F: JamsocketServiceFactory<ServiceActorContext>>(
         Err(e) => Err(e),
     }
 }
+
+async fn status(req: HttpRequest) -> Result<web::Json<ConnectionInfo>, Error> {
+    let server_state: &Data<ServerState> = req.app_data().expect("Could not load ServerState.");
+
+    let room_addr = server_state.room_addr.clone();
+    let connection_info = room_addr
+        .send(GetConnectionInfo)
+        .await
+        .map_err(|_| ErrorInternalServerError("Error getting connection info."))?;
+
+    Ok(web::Json(connection_info))
+}
