@@ -3,7 +3,7 @@ use actix::{
     prelude::SendError, Actor, AsyncContext, Context, Handler, Message, Recipient, SpawnHandle,
 };
 use jamsocket::{JamsocketContext, JamsocketService, JamsocketServiceFactory, MessageRecipient};
-use std::{sync::Arc, time::Duration};
+use std::time::Duration;
 
 pub struct ServiceActor<J: JamsocketService> {
     service: J,
@@ -70,17 +70,18 @@ impl JamsocketContext for ServiceActorContext {
 }
 
 impl<J: JamsocketService> ServiceActor<J> {
+    #[must_use]
     pub fn new(
         ctx: &Context<Self>,
-        room_id: &str,
-        service_constructor: Arc<impl JamsocketServiceFactory<ServiceActorContext, Service = J>>,
+        service_factory: impl JamsocketServiceFactory<ServiceActorContext, Service = J>,
         recipient: Recipient<MessageFromServer>,
     ) -> Option<Self> {
         let host_context = ServiceActorContext {
             set_timer_recipient: ctx.address().recipient(),
             send_message_recipient: recipient,
         };
-        let service = service_constructor.build(room_id, host_context)?;
+
+        let service = service_factory.build("", host_context).unwrap();
 
         Some(ServiceActor {
             service,
